import(methods)
importFrom(stringr, str_locate, str_match, str_split_fixed, str_trim)
importFrom(R6, R6Class)

export(css_to_xpath)
export(querySelector)
export(querySelectorAll)
export(querySelectorNS)
export(querySelectorAllNS)

S3method(querySelector, default)
S3method(querySelectorAll, default)
S3method(querySelectorNS, default)
S3method(querySelectorAllNS, default)
